------------------------------------------------------
-- FactionFriend.lua
------------------------------------------------------

FFF_Config = {};
FFF_LastBarSwitchTime = 0;

FFF_TooltipHooks = {};
function FFF_GameTooltip_OnHide()
	local tooltipName = this:GetName();
	FFF_SetTooltipHooks(tooltipName);
end

function FFF_GameTooltip_SetDefaultAnchor(tooltip, parent)
	local tooltipName = tooltip:GetName();
	FFF_SetTooltipHooks(tooltipName);
end

function FFF_SetTooltipHooks(tooltipName)
	if (not tooltipName or tooltipName == "") then return; end
	local tooltip = getglobal(tooltipName);
	if (tooltip and tooltip:HasScript("OnTooltipSetItem") and not FFF_TooltipHooks[tooltipName]) then
		FFF_TooltipHooks[tooltipName] = {};
		FFF_TooltipHooks[tooltipName].OnTooltipSetItem = tooltip:GetScript("OnTooltipSetItem");
		tooltip:SetScript("OnTooltipSetItem", FFF_OnTooltipSetItem);
	end
end

function FFF_OnTooltipSetItem()
	local tooltipName = this:GetName();
	local tooltip = getglobal(tooltipName);
	if (FFF_TooltipHooks[tooltipName] and FFF_TooltipHooks[tooltipName].OnTooltipSetItem) then
		FFF_TooltipHooks[tooltipName].OnTooltipSetItem(tooltip);
	end
	
	local name, link = this:GetItem();
	if (not link) then return; end
	if (not FFF_Config.NoTooltip and link) then
		local _, _, itemID  = string.find(link, "item:(%d+)");
		itemID = tonumber(itemID);
		
		if (FFF_ReverseCache == nil) then
			FFF_SetupReverseCache();
		end
		
		local itemInfo = FFF_ReverseCache[itemID];
		if (not FFF_Config.NoDarkmoon and itemInfo) then
			if (type(itemInfo) == "table") then
				FFF_AddTooltipLine(this, FFF_FACTION_TURNIN..":", itemInfo[1]);
				for i = 2, #itemInfo do
					FFF_AddTooltipLine(this, " ", itemInfo[i]);
				end
			else
				FFF_AddTooltipLine(this, FFF_FACTION_TURNIN, itemInfo, 1);
			end
			return true;
		end

	end
	
end

function FFF_AddTooltipLine(tooltip, leftText, faction, isOnlyLine)
	local factionText = faction;
	if (faction == FFF_RACIAL_FACTIONS) then
		faction = UnitFactionGroup("player");
		factionText = faction..FFF_RACIAL_FACTIONS;
	end
	if (FFF_Notes[itemID] and FFF_Notes[itemID][faction]) then
		factionText = factionText.." ("..FFF_Notes[itemID][faction]..")";
	end
	local _, _, standingID = FFF_GetFactionInfoByName(faction);
	if (standingID == nil) then
		standingID = 4;
	end
	local baseColor = FACTION_BAR_COLORS[standingID];
	local color = {};
	for key, value in pairs(baseColor) do 
		color[key] = math.min(1, value * 1.25);
	end
	if (isOnlyLine) then
		this:AddLine(leftText.." "..factionText, color.r, color.g, color.b);
	else
		local c = HIGHLIGHT_FONT_COLOR;
		this:AddDoubleLine(leftText, factionText, c.r, c.g, c.b, color.r, color.g, color.b);
	end
end

function FFF_OnLoad()

	hooksecurefunc("ReputationWatchBar_Update", FFF_ReputationWatchBar_Update);
	
	FFF_Orig_ReputationWatchBar_OnEnter = ReputationWatchBar:GetScript("OnEnter");
	ReputationWatchBar:SetScript("OnEnter", FFF_ReputationWatchBar_OnEnter);
	FFF_Orig_ReputationWatchBar_OnLeave = ReputationWatchBar:GetScript("OnLeave");
	ReputationWatchBar:SetScript("OnLeave", FFF_ReputationWatchBar_OnLeave);
		
	this:RegisterEvent("UPDATE_FACTION");
	this:RegisterEvent("PLAYER_ENTERING_WORLD");
	this:RegisterEvent("PLAYER_LEAVING_WORLD");
	this:RegisterEvent("PLAYER_CONTROL_GAINED");
	this:RegisterEvent("CHAT_MSG_COMBAT_FACTION_CHANGE");
	this:RegisterEvent("ZONE_CHANGED");
	this:RegisterEvent("ZONE_CHANGED_NEW_AREA");
	
	table.insert(UISpecialFrames,"FFF_OptionsFrame");	

	-- tooltip hooks
	hooksecurefunc("GameTooltip_OnHide", FFF_GameTooltip_OnHide);
	hooksecurefunc("GameTooltip_SetDefaultAnchor", FFF_GameTooltip_SetDefaultAnchor);
	FFF_SetTooltipHooks("GameTooltip");
	FFF_SetTooltipHooks("ItemRefTooltip");
	FFF_SetTooltipHooks("ShoppingTooltip1");
	FFF_SetTooltipHooks("ShoppingTooltip2");
	
	-- Register Slash Commands
	SLASH_FFF1 = "/factionfriend";
	SLASH_FFF2 = "/ff";
	SlashCmdList["FFF"] = function(msg)
		if FFF_OptionsFrame:IsVisible() then
			HideUIPanel(FFF_OptionsFrame);
		else
			ShowUIPanel(FFF_OptionsFrame);
		end
	end
	
	local version = GetAddOnMetadata("GFW_FactionFriend", "Version");
	GFWUtils.Print("Fizzwidget FactionFriend "..version.." initialized!");
	
end

function FFF_OnEvent(event, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9)
	if ( event == "PLAYER_ENTERING_WORLD" ) then
		this:RegisterEvent("BAG_UPDATE");
		this:RegisterEvent("UNIT_INVENTORY_CHANGED");
	elseif( event == "PLAYER_LEAVING_WORLD" ) then
		this:UnregisterEvent("UNIT_INVENTORY_CHANGED");
		this:RegisterEvent("BAG_UPDATE");
	elseif( event == "PLAYER_CONTROL_GAINED" ) then
		if (FFF_OnTaxi) then
			FFF_OnTaxi = nil;
			FFF_CheckZone();
		end
	elseif ( event == "ZONE_CHANGED_NEW_AREA" or event == "ZONE_CHANGED" ) then
		if (UnitOnTaxi("player")) then
			FFF_OnTaxi = 1;
			return;
		end
		FFF_CheckZone();
	elseif ( event == "CHAT_MSG_COMBAT_FACTION_CHANGE" ) then
		if (FFF_Config.NoRepGained) then return; end
		
		local currentZone = GetRealZoneText();
		local zoneFaction = FFF_ZoneFactions[UnitFactionGroup("player")][currentZone] or FFF_ZoneFactions.Neutral[currentZone];
		if (FFF_FACTION_STANDING_INCREASED == nil) then
			FFF_FACTION_STANDING_INCREASED = GFWUtils.FormatToPattern(FACTION_STANDING_INCREASED);
		end
		local _, _, factionName = string.find(arg1, FFF_FACTION_STANDING_INCREASED);
		if (FFF_Config.NoZones) then
			local watchedFaction = GetWatchedFactionInfo();
			if (watchedFaction ~= factionName and GetTime() - FFF_LastBarSwitchTime > 5) then
				FFF_SetWatchedFaction(factionName);
			end
		else
			if (not zoneFaction or zoneFaction == factionName) then
				local watchedFaction = GetWatchedFactionInfo();
				if (watchedFaction ~= factionName and GetTime() - FFF_LastBarSwitchTime > 5) then
					FFF_SetWatchedFaction(factionName);
				end
			end
		end
	elseif ( event == "BAG_UPDATE") then
		FFF_ReputationWatchBar_Update();
	elseif ( event == "UNIT_INVENTORY_CHANGED") then
		if (arg1 == "player") then
			FFF_ReputationWatchBar_Update();
		end
	end
end

local racialFactions = {
	["Alliance"] = { FACTION_DWARF, FACTION_NELF, FACTION_GNOME, FACTION_HUMAN, FACTION_DRAENEI },
	["Horde"] = { FACTION_ORC, FACTION_TAUREN, FACTION_TROLL, FACTION_FORSAKEN, FACTION_BELF },
};
function FFF_SetupReverseCache()
	FFF_ReverseCache = {};
	FFF_Notes = {};
	for faction, quests in pairs(FFF_ItemInfo) do
		local myExcludedFactions = FFF_ExcludedFactions[UnitFactionGroup("player")];
		if (GFWTable.KeyOf(myExcludedFactions, faction) == nil) then
			for quest, questInfo in pairs(quests) do						
				for itemID in pairs(questInfo.items) do
					if (FFF_ReverseCache[itemID]) then
						if (type(FFF_ReverseCache[itemID]) == "table") then
							if (GFWTable.KeyOf(FFF_ReverseCache[itemID], faction) == nil) then
								table.insert(FFF_ReverseCache[itemID], faction);
							end
						elseif (FFF_ReverseCache[itemID] ~= faction) then
							local oldValue = FFF_ReverseCache[itemID];
							FFF_ReverseCache[itemID] = {};
							table.insert(FFF_ReverseCache[itemID], oldValue);
							table.insert(FFF_ReverseCache[itemID], faction);
						end
					else
						FFF_ReverseCache[itemID] = faction;
					end
					if (questInfo.note) then
						if (FFF_Notes[itemID] == nil) then
							FFF_Notes[itemID] = {};
						end
						FFF_Notes[itemID][faction] = questInfo.note;
					end
				end
			end
		end
	end
	local myRacialFactions = racialFactions[UnitFactionGroup("player")];
	for itemID, factionInfo in pairs(FFF_ReverseCache) do
		if (type(factionInfo) == "table") then
			local racialSubtracted = GFWTable.Subtract(factionInfo, myRacialFactions);
			if (#factionInfo - #racialSubtracted == #myRacialFactions) then
				if (#racialSubtracted == 0) then
					FFF_ReverseCache[itemID] = FFF_RACIAL_FACTIONS;
				else
					FFF_ReverseCache[itemID] = racialSubtracted;
					table.insert(FFF_ReverseCache[itemID], FFF_RACIAL_FACTIONS);
				end
			end
		end
	end
end

function FFF_GetFactionInfoByName(factionName)
	for factionIndex = 1, GetNumFactions() do
		local name, description, standingID, bottomValue, topValue, earnedValue, atWarWith, canToggleAtWar, isHeader, isCollapsed, isWatched = GetFactionInfo(factionIndex)
		if (name == factionName) then
			return name, description, standingID, bottomValue, topValue, earnedValue, atWarWith, canToggleAtWar, isHeader, isCollapsed, isWatched;
		end
	end
end

function FFF_CheckZone()
	if (FFF_Config.NoZones) then return; end
	
	local currentZone = GetRealZoneText();
	local zoneFaction = FFF_ZoneFactions[UnitFactionGroup("player")][currentZone] or FFF_ZoneFactions.Neutral[currentZone];
	if (zoneFaction) then
		local watchedFaction = GetWatchedFactionInfo();
		if (watchedFaction ~= zoneFaction and GetTime() - FFF_LastBarSwitchTime > 5) then
			FFF_SetWatchedFaction(zoneFaction);
		end
	end
end

function FFF_SetWatchedFaction(factionName)
	for index = 1, GetNumFactions() do
		local name = GetFactionInfo(index);
		if (name == factionName and not IsFactionInactive(index)) then
			SetWatchedFactionIndex(index);
			FFF_LastBarSwitchTime = GetTime();
			break;
		end
	end
end

function FFF_IsFactionItem(itemID)
	for factionName, factionInfo in pairs(FFF_ItemInfo) do
		for quest, qInfo in pairs(factionInfo) do
			for id, qtyPerTurnin in pairs(qInfo.items) do
				if (id == itemID) then 
					return true; 
				end
			end
		end
	end
end

function FFF_GetWatchedFactionPotential(withReport)

	local factionName, standing, min, max, value = GetWatchedFactionInfo();
	local factionInfo = FFF_ItemInfo[factionName];
	if (factionInfo == nil) then return 0; end
	
	local totalPotential = 0;
	local reportLines;
	if (withReport) then
		reportLines = {};
	end
	local multiplier = 1;
	local _, race = UnitRace("player");
	if (race == "Human") then
		multiplier = 1.1;
	end
	local itemsAccountedFor = {};
	for quest, qInfo in pairs(factionInfo) do
		if ((qInfo.minStanding and standing >= qInfo.minStanding) or not qInfo.minStanding) then
			if ((qInfo.maxStanding and standing <= qInfo.maxStanding) or not qInfo.maxStanding) then
				local potentialValue = 0;
				
				local turninCounts = {};
				local reportItemLines = {};
				for itemID, qtyPerTurnin in pairs(qInfo.items) do
					local countIncludingBank = GetItemCount(itemID, true);
					local alreadyCounted = itemsAccountedFor[itemID] or 0;
					local turnins = math.floor((countIncludingBank - alreadyCounted) / qtyPerTurnin);
					itemsAccountedFor[itemID] = turnins * qtyPerTurnin;
					table.insert(turninCounts, turnins);
					if (withReport and turnins > 0) then
						local itemLink = GFWUtils.ItemLink(itemID);
						local lineItem = string.format(FFF_REPORT_LINE_ITEM, turnins * qtyPerTurnin, itemLink);
						local countInBags = GetItemCount(itemID, false);
						if (countIncludingBank and countIncludingBank - countInBags > 0 and countInBags < turnins * qtyPerTurnin) then
							lineItem = lineItem ..string.format(FFF_COUNT_IN_BANK, math.min(countIncludingBank - countInBags, turnins * qtyPerTurnin));
						end
						table.insert(reportItemLines, lineItem);
					end
				end
				--DevTools_Dump(turninCounts);
				
				local numTurnins;
				local reportLineHeader;
				if (table.getn(turninCounts) == 1) then
					numTurnins = turninCounts[1];
				elseif (table.getn(turninCounts) > 1) then
					numTurnins = math.min(unpack(turninCounts));
				else
					numTurnins = 0;
				end
				potentialValue = numTurnins * qInfo.value * multiplier;
				
				if (qInfo.maxValue and standing == qInfo.maxStanding) then
					if (value >= qInfo.maxValue) then
						potentialValue = 0;
					end
					local maxPotential = qInfo.maxValue - value;
					potentialValue = math.min(potentialValue, maxPotential);
				end
				if (potentialValue > 0 and withReport) then
					local reportLineHeader = string.format(FFF_REPORT_NUM_POINTS, potentialValue);
					if (numTurnins > 1 and not qInfo.useItem) then
						reportLineHeader = reportLineHeader ..string.format(FFF_REPORT_NUM_TURNINS, numTurnins);
					end
					reportLineHeader = reportLineHeader ..":";
					table.insert(reportLines, {[reportLineHeader] = reportItemLines});
				end
				
				totalPotential = totalPotential + potentialValue;
			end
		end
	end
	return totalPotential, reportLines;

end

function FFF_ReputationWatchBar_Update(newLevel)
	
	local name, reaction, min, max, value = GetWatchedFactionInfo();
	if (not name) then return; end
	
	-- Normalize values
	max = max - min;
	value = value - min;
	min = 0;

	local itemsValue = FFF_GetWatchedFactionPotential();
	local tickSet = math.max(((value + itemsValue) / max) * ReputationWatchStatusBar:GetWidth(), 0);
	FFF_ReputationTick:ClearAllPoints();
	if (tickSet > ReputationWatchStatusBar:GetWidth() or itemsValue == 0 or FFF_Config.NoShowPotential) then
	    FFF_ReputationTick:Hide();
	    FFF_ReputationExtraFillBarTexture:Hide();
	else
	    FFF_ReputationTick:Show();
	    FFF_ReputationTick:SetPoint("CENTER", "ReputationWatchStatusBar", "LEFT", tickSet, 0);
	    local color = FACTION_BAR_COLORS[reaction];
	    FFF_ReputationExtraFillBarTexture:Show();
	    FFF_ReputationExtraFillBarTexture:SetPoint("TOPRIGHT", "ReputationWatchStatusBar", "TOPLEFT", tickSet, 0);
	    FFF_ReputationExtraFillBarTexture:SetVertexColor(color.r, color.g, color.b, 0.15);
		FFF_ReputationTickHighlight:SetVertexColor(color.r, color.g, color.b);
	end

end

function FFF_ReputationTick_Tooltip()

	local x,y;
	x,y = FFF_ReputationTick:GetCenter();
	if ( FFF_ReputationTick:IsVisible() ) then
		FFF_ReputationTick:LockHighlight();
		if ( x >= ( GetScreenWidth() / 2 ) ) then
			GameTooltip:SetOwner(FFF_ReputationTick, "ANCHOR_LEFT");
		else
			GameTooltip:SetOwner(FFF_ReputationTick, "ANCHOR_RIGHT");
		end
	else
		GameTooltip_SetDefaultAnchor(GameTooltip, UIParent);
	end
	
	local potential, reportLines = FFF_GetWatchedFactionPotential(1);
	local tooltipText = string.format(REPUTATION_TICK_TOOLTIP, potential);
	local name, reaction, min, max, value = GetWatchedFactionInfo();
    local color = FACTION_BAR_COLORS[reaction];
	
	local standingName = getglobal("FACTION_STANDING_LABEL"..reaction);
	if (UnitSex("player") == 3) then
		local standingName = getglobal("FACTION_STANDING_LABEL"..reaction.."_FEMALE");
	end
	GameTooltip:SetText(name..": "..standingName, color.r, color.g, color.b);

	if (potential == 0) then return; end

	GameTooltip:AddLine(tooltipText);
	for _, reportLine in pairs(reportLines) do
		for reportHeader, itemLines in pairs(reportLine) do
			local r, g, b = HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b;
			GameTooltip:AddDoubleLine(reportHeader, itemLines[1], r, g, b, r, g, b);
			if (table.getn(itemLines) > 1) then
				for i = 2, table.getn(itemLines) do
					GameTooltip:AddDoubleLine(" ", itemLines[i], r, g, b, r, g, b);
				end
			end
		end
	end
	GameTooltip:Show();

end

function FFF_ReputationWatchBar_OnEnter()
	FFF_Orig_ReputationWatchBar_OnEnter();
	if (FFF_Config.NoShowPotential) then return; end
	FFF_ReputationTick_Tooltip();
	FFF_ShowingTooltip = true;
end

function FFF_ReputationWatchBar_OnLeave()
	FFF_Orig_ReputationWatchBar_OnLeave();
	FFF_ReputationTick:UnlockHighlight();
	if (FFF_ShowingTooltip) then
		GameTooltip:Hide();
		FFF_ShowingTooltip = false;
	end
end

-- Options window
-- All options are the inverse of their checkboxes; this way we store nothing in SV if default.
FFF_OptionsText = {
	["Zones"] = FFF_OPTION_ZONES,
	["RepGained"] = FFF_OPTION_REP_GAINED,
	["ShowPotential"] = FFF_OPTION_SHOW_POTENTIAL,
	["Tooltip"] = FFF_OPTION_TOOLTIP,
};

function FFF_OptionsShow()

	local version = GetAddOnMetadata("GFW_FactionFriend", "Version");
	FFF_VersionText:SetText("v. "..version);

	for option, description in pairs(FFF_OptionsText) do
		local button = getglobal("FFF_OptionsButton_"..option);
		local text = getglobal("FFF_OptionsButton_"..option.."Text");
		
		if (button and text) then
			button:SetChecked(not FFF_Config["No"..option]);
			text:SetText(description);
		end
	end
	
end

function FFF_OptionsClick()
	local button = this:GetName();
	local option = string.gsub(button, "FFF_OptionsButton_", "");
	FFF_Config["No"..option] = not this:GetChecked();
end


-- ReputationWatchBar menu shamelessly, er, borrowed from AnduinLothar. Thanks!

function FFF_ToggleMenu()
	FFF_MenuDropDown.displayMode = "MENU";
	ToggleDropDownMenu(1, nil, FFF_MenuDropDown, "cursor", 0, 0, "TOP");
	PlaySound("igMainMenuOptionCheckBoxOn");
end

function FFF_ReputationWatchBar_OnClick(button)
	if (button == "RightButton") then
		FFF_ToggleMenu();
	end
end

function FFF_LoadMenu()
		
	local gender = UnitSex("player");
	local info;
	for i = 1, GetNumFactions() do
		if (not IsFactionInactive(i)) then
			local name, description, standingID, barMin, barMax, barValue, atWarWith, canToggleAtWar, isHeader, isCollapsed, isWatched = GetFactionInfo(i);
			
			if (not (isHeader and name == FACTION_INACTIVE)) then
				info = {};
				info.value = i;
				if (isHeader) then
					info.text = name;
					info.notClickable = 1;
					info.notCheckable = 1;
					info.isTitle = isHeader;
				else
					local baseColor = FACTION_BAR_COLORS[standingID];
					local color = {};
					for key, value in pairs(baseColor) do 
						color[key] = math.min(1, value * 1.25);
					end
					local standingLabel = GetText("FACTION_STANDING_LABEL"..standingID, gender);
					info.text = name..GFWUtils.ColorText(" ("..standingLabel..")", color);
					if (isWatched) then
						info.func = function()
							SetWatchedFactionIndex(0);
						end
					else
						info.func = function()
							SetWatchedFactionIndex(info.value);
						end
					end
				end
				info.checked = isWatched;
				UIDropDownMenu_AddButton(info, 1);
			end
		end
	end
	
	-- spacer
	info = {};
	info.disabled = 1;
	info.text = "";
	UIDropDownMenu_AddButton(info);
	
	info = {};
	info.text = FFF_SHOW_REPUTATION_PANE;
	info.func = function() 
		ToggleCharacter("ReputationFrame"); 
	end;
	info.notCheckable = 1;
	UIDropDownMenu_AddButton(info);

	info = {};
	info.text = FFF_SHOW_OPTIONS;
	info.func = function()
		if FFF_OptionsFrame:IsVisible() then
			HideUIPanel(FFF_OptionsFrame);
		else
			ShowUIPanel(FFF_OptionsFrame);
		end
	end;
	info.notCheckable = 1;
	UIDropDownMenu_AddButton(info);
	
	
	-- nil the displayMode so that the rest of ToggleDropDownMenu doesn't error on the fake frame
	FFF_MenuDropDown.displayMode = nil;
end

-- Fake menu frame.... who says you need a frame, bah!
FFF_MenuDropDown = {
	initialize = FFF_LoadMenu;
	GetName = function() return "FFF_MenuDropDown" end;
	SetHeight = function() end;
}

local oldFunc = ReputationWatchBar:GetScript("OnMouseDown");
if (oldFunc) then
	ReputationWatchBar:SetScript("OnMouseDown", function() oldFunc(); FFF_ReputationWatchBar_OnClick(arg1); end);
else
	ReputationWatchBar:SetScript("OnMouseDown", function() FFF_ReputationWatchBar_OnClick(arg1); end);
end

